from .client import Client
from .error import ClueaiError

MODELFUN_API_URL = 'https://www.modelfun.cn/modelfun/api/'
TEXT_TO_IMAGE_URL= "http://modelfun.westlake.ink:6001/clueai/text2image?text="
MODELFUN_VERSION = '2022-08-08'
GENERATE_URL = 'serving_api'
CLASSIFY_URL = 'serving_api'
EXTRACT_URL = 'extract'
CHECK_API_KEY_URL = 'check_api_key'
TOKENIZE_URL = 'tokenize'
